/********************
 * 2004 Jalview Reengineered
 * Barton Group
 * Dundee University
 *
 * AM Waterhouse
 *******************/


package jalview.gui;

import jalview.gui.*;
import jalview.io.*;
import jalview.datamodel.*;
import jalview.schemes.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


public class Desktop extends jalview.jbgui.GDesktop
{
  public static JDesktopPane desktop;
  static int openFrameCount = 0;
  static final int xOffset = 30, yOffset = 30;

  public Desktop()
  {
    Image image =null;
    try{
      java.net.URL url = getClass().getResource("/images/logo.gif");
      if(url!=null)
      {
        image = java.awt.Toolkit.getDefaultToolkit().createImage(url);
        MediaTracker mt = new MediaTracker(this);
        mt.addImage(image, 0);
        mt.waitForID(0);
        setIconImage(image);
      }

    }catch(Exception ex){}

    setTitle("Jalview 2005");
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    setVisible(true);
    desktop = new JDesktopPane();
    desktop.setBackground(Color.white);
    setContentPane(desktop);
    desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);

    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    String x = jalview.bin.Cache.getProperty("SCREEN_X");
    String y = jalview.bin.Cache.getProperty("SCREEN_Y");
    String width = jalview.bin.Cache.getProperty("SCREEN_WIDTH");
    String height = jalview.bin.Cache.getProperty("SCREEN_HEIGHT");

    if(x!=null && y!=null && width!=null && height!=null)
    {
      setBounds( Integer.parseInt(x), Integer.parseInt(y),
                 Integer.parseInt(width), Integer.parseInt(height));
    }
    else
      setBounds( (int)(screenSize.width-900)/2,
           (int)(screenSize.height-650)/2,
          900 ,
          650);

this.addWindowListener(new WindowAdapter()
{
public void windowClosing(WindowEvent evt)
{
 jalview.bin.Cache.setProperty("SCREEN_X",
                               getBounds().x + "");
 jalview.bin.Cache.setProperty("SCREEN_Y",
                               getBounds().y + "");
 jalview.bin.Cache.setProperty("SCREEN_WIDTH", getWidth() + "");
 jalview.bin.Cache.setProperty("SCREEN_HEIGHT", getHeight() + "");
}
});

/////////Add a splashscreen on startup


   /////////Add a splashscreen on startup
    JInternalFrame frame = new JInternalFrame();

    SplashScreen splash = new SplashScreen(frame, image);
    frame.setContentPane(splash);
    desktop.add(frame);
    frame.setVisible(true);
    openFrameCount++;
    try
    {
      frame.setSelected(true);
    }
    catch (java.beans.PropertyVetoException e)
    {}
    frame.setResizable(true);
    frame.setSize(750, 160);
    frame.setLocation( (int)((getWidth()-750) /2), (int)((getHeight()-160) /2));
    frame.setClosable(false);
    frame.setIconifiable(false);
    frame.setMaximizable(false);
    frame.setFrameIcon(null);

  }

  public static void addInternalFrame(final JInternalFrame frame, String title, int w, int h)
  {
   desktop.add(frame);
   frame.setVisible(true);
   openFrameCount++;
   try {
       frame.setSelected(true);
   } catch (java.beans.PropertyVetoException e) {}
   frame.setTitle(title);
   frame.setResizable(true);
   frame.setSize(w,h);
   frame.setClosable(true);
   frame.setMaximizable(true);
   frame.setIconifiable(true);
   frame.setFrameIcon(null);
   frame.setLocation(xOffset*openFrameCount, yOffset*openFrameCount);
   frame.toFront();
   final JMenuItem menuItem = new JMenuItem(title);
   frame.addInternalFrameListener(new javax.swing.event.InternalFrameAdapter()
                                   { public void internalFrameClosed(javax.swing.event.InternalFrameEvent evt)
                                     {
                                       openFrameCount--;
                                       windowMenu.remove(menuItem);
                                     };
                                   });

   menuItem.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        try{  frame.setSelected(true);   frame.setIcon(false); }
        catch(java.beans.PropertyVetoException ex){};
      }
    });

   windowMenu.add(menuItem);
 }



 public void inputLocalFileMenuItem_actionPerformed(ActionEvent e)
 {
   JalviewFileChooser chooser = new JalviewFileChooser(jalview.bin.Cache.getProperty("LAST_DIRECTORY"),
       new String[]{"fa", "fasta", "fastq", "blc", "msf", "pfam", "aln", "pir"}, "Alignment files");
   chooser.setFileView(new JalviewFileView());
   chooser.setDialogTitle("Open local file");
   chooser.setToolTipText("Open");
   int value = chooser.showOpenDialog(this);
   if(value == JalviewFileChooser.APPROVE_OPTION)
   {
     String choice =  chooser.getSelectedFile().getPath();
     jalview.bin.Cache.setProperty("LAST_DIRECTORY", choice);
     String format = IdentifyFile.Identify(choice, "File");
     LoadFile(choice, "File", format);
   }
 }

 public void LoadFile(String file, String protocol,  String format)
 {
   SequenceI [] sequences = null;

   if (FormatProperties.contains(format))
       sequences = FormatAdapter.read(file, protocol, format);

   if (sequences != null)
   {
     AlignFrame af = new AlignFrame(new Alignment(sequences));
     addInternalFrame(af, file, 700, 500);
     af.statusBar.setText("Successfully loaded file " + file);


   }
   else
     JOptionPane.showInternalMessageDialog(Desktop.desktop,  "Couldn't open file.\n"
                          + "Formats currently supported are\n"
                          + "Fasta, MSF, Clustal, BLC, PIR, MSP or PFAM"
                          ,"Error loading file",
                                   JOptionPane.WARNING_MESSAGE);

 }

 public void inputURLMenuItem_actionPerformed(ActionEvent e)
 {
   String url = JOptionPane.showInternalInputDialog(Desktop.desktop,"Enter url of input file",
                                            "Input alignment from URL",
                                            JOptionPane.QUESTION_MESSAGE);
   if (url == null)
     return;

   String format = IdentifyFile.Identify(url, "URL");

   System.out.println(format +" format");
   if (format.equals("URL NOT FOUND"))
   {
     JOptionPane.showInternalMessageDialog(Desktop.desktop,"Couldn't locate " + url,
                                   "URL not found",
                                   JOptionPane.WARNING_MESSAGE);
     return;
   }

    LoadFile(url, "URL", format);
 }

 public void inputTextboxMenuItem_actionPerformed(ActionEvent e)
 {
   CutAndPasteTransfer cap = new CutAndPasteTransfer(true);
   int accept =  JOptionPane.showInternalOptionDialog(Desktop.desktop, cap, "Cut & paste Alignment File",
                                       JOptionPane.YES_NO_CANCEL_OPTION,
                                       JOptionPane.PLAIN_MESSAGE,
                                       null,
                                       new Object[]{"Accept", "Cancel", }, null);


  if(accept == JOptionPane.YES_OPTION)
  {
    String format = IdentifyFile.Identify(cap.getText(), "Paste");
    SequenceI [] sequences = null;

    if (FormatProperties.contains( format ))
      sequences = FormatAdapter.read(cap.getText(), "Paste", format);


      if(sequences!=null)
      {
        AlignFrame af = new AlignFrame(new Alignment(sequences));
        addInternalFrame(af, "Cut & Paste input - "+format, 700, 500);
        af.statusBar.setText("Successfully pasted alignment file");
      }
      else
        JOptionPane.showInternalMessageDialog(Desktop.desktop, "Couldn't read the pasted text.\n"
                                      +"Formats currently supported are\n"
                                      +"Fasta, MSF, Clustal, BLC, PIR, MSP or PFAM",
                                      "Error parsing text", JOptionPane.WARNING_MESSAGE);

   }
 }

/*
* Exit the program
*/
 public void quit_actionPerformed(ActionEvent e)
 {
   if(jalview.bin.Jalview.applet!=null)
     jalview.bin.Jalview.applet.destroy();
    else
      System.exit(0);
 }



 public void aboutMenuItem_actionPerformed(ActionEvent e)
 {
   JOptionPane.showInternalMessageDialog(Desktop.desktop,
                                         "JalView 2005 version " + jalview.bin.Cache.VERSION+"; last updated: "+jalview.bin.Cache.BUILD_DATE
                                       +"\nAuthors:  Michele Clamp, James Cuff, Steve Searle, Andrew Waterhouse, Jim Procter & Geoff Barton."
                                       +"\nCurrent development managed by Andrew Waterhouse; Barton Group, University of Dundee."
                                       +"\nIf  you use JalView, please cite:  \"Clamp, M., Cuff, J., Searle, S. M. and Barton, G. J. (2004), The Jalview Java Alignment Editor\"",
                                         "About Jalview",
                                         JOptionPane.INFORMATION_MESSAGE);
 }

 public void documentationMenuItem_actionPerformed(ActionEvent e)
 {
   try
   {
      ClassLoader cl = jalview.gui.Desktop.class.getClassLoader();
      java.net.URL url = javax.help.HelpSet.findHelpSet(cl,"help/help");
      javax.help.HelpSet hs = new javax.help.HelpSet(cl,url);

      javax.help.HelpBroker hb = hs.createHelpBroker();
      hb.setLocation(new Point(200, 50));
      hb.setSize(new Dimension(800,700));
      hb.setCurrentID("home");
      hb.setDisplayed(true);
   }
   catch (Exception ex)
   {
     ex.printStackTrace();
   }
 }

 static JInternalFrame conservationSlider;
 static JInternalFrame PIDSlider;
 public static int setConservationSliderSource(AlignmentPanel ap, ColourSchemeI cs, String source)
 {
   SliderPanel sp = null;

   if(conservationSlider == null)
   {
     sp = new SliderPanel(ap, 30, true, cs);
     conservationSlider = new JInternalFrame();
     conservationSlider.setContentPane(sp);
     conservationSlider.setResizable(false);
     addInternalFrame(conservationSlider, "Conservation Colour Increment  ("+source+")", 420, 90);
     conservationSlider.addInternalFrameListener(new javax.swing.event.InternalFrameAdapter()
                  {
                    public void internalFrameClosed(javax.swing.event.InternalFrameEvent e)
                    {
                      conservationSlider = null;
                    }
                  });
   }
   else
   {
       conservationSlider.setTitle("Conservation Colour Increment  ("+source+")");
       sp = (SliderPanel)conservationSlider.getContentPane();
       sp.cs = cs;
   }

   if(ap.av.alignment.getGroups()!=null)
     sp.setAllGroupsCheckEnabled( true );
   else
     sp.setAllGroupsCheckEnabled( false);

   return sp.getValue();

 }

 public static void hideConservationSlider()
 {
  try{
    conservationSlider.setClosed(true);
    conservationSlider = null;
  }catch(Exception ex){}
}


 public static void hidePIDSlider()
 {
   try{
     PIDSlider.setClosed(true);
     PIDSlider = null;
   }catch(Exception ex){}
 }

 public static int setPIDSliderSource(AlignmentPanel ap, ColourSchemeI cs, String source)
 {
   SliderPanel pid = null;
   if(PIDSlider == null)
   {
     pid = new SliderPanel(ap, 50, false, cs);
     PIDSlider = new JInternalFrame();
     PIDSlider.setContentPane(pid);
     PIDSlider.setResizable(false);
     addInternalFrame(PIDSlider, "Percentage Identity Threshold ("+source+")", 420, 90);
   }
   else
   {
       PIDSlider.setTitle("Percentage Identity Threshold ("+source+")");
       pid = (SliderPanel)PIDSlider.getContentPane();
       pid.cs = cs;
   }

   if (ap.av.alignment.getGroups() != null)
     pid.setAllGroupsCheckEnabled(true);
   else
     pid.setAllGroupsCheckEnabled(false);


   return pid.getValue();

 }

}
